LIBRARY     LCMS2.DLL

EXPORTS

_cms15Fixed16toDouble                    =   _cms15Fixed16toDouble
_cms8Fixed8toDouble                      =   _cms8Fixed8toDouble
cmsAdaptToIlluminant                     =    cmsAdaptToIlluminant
_cmsAdjustEndianess16                    =   _cmsAdjustEndianess16
_cmsAdjustEndianess32                    =   _cmsAdjustEndianess32
_cmsAdjustEndianess64                    =   _cmsAdjustEndianess64
cmsAllocNamedColorList                   =   cmsAllocNamedColorList
cmsAllocProfileSequenceDescription       =   cmsAllocProfileSequenceDescription
cmsAppendNamedColor                      =   cmsAppendNamedColor
cmsBFDdeltaE                             =   cmsBFDdeltaE
cmsBuildGamma                            =   cmsBuildGamma
cmsBuildParametricToneCurve              =   cmsBuildParametricToneCurve
cmsBuildSegmentedToneCurve               =   cmsBuildSegmentedToneCurve
cmsBuildTabulatedToneCurve16             =   cmsBuildTabulatedToneCurve16
cmsBuildTabulatedToneCurveFloat          =   cmsBuildTabulatedToneCurveFloat
_cmsCalloc                               =   _cmsCalloc
cmsChannelsOf                            =    cmsChannelsOf
cmsCIE2000DeltaE                         =    cmsCIE2000DeltaE
cmsCIE94DeltaE                           =    cmsCIE94DeltaE
cmsCIECAM02Done                          =    cmsCIECAM02Done
cmsCIECAM02Forward                       =    cmsCIECAM02Forward
cmsCIECAM02Init                          =    cmsCIECAM02Init
cmsCIECAM02Reverse                       =    cmsCIECAM02Reverse
cmsCloseIOhandler                        =    cmsCloseIOhandler
cmsCloseProfile                          =    cmsCloseProfile
cmsCMCdeltaE                             =    cmsCMCdeltaE
cmsCreate_sRGBProfile                    =    cmsCreate_sRGBProfile
cmsCreate_sRGBProfileTHR                 =    cmsCreate_sRGBProfileTHR
cmsCreateBCHSWabstractProfile            =    cmsCreateBCHSWabstractProfile
cmsCreateBCHSWabstractProfileTHR         =    cmsCreateBCHSWabstractProfileTHR
cmsCreateExtendedTransform               =    cmsCreateExtendedTransform
cmsCreateGrayProfile                     =    cmsCreateGrayProfile
cmsCreateGrayProfileTHR                  =    cmsCreateGrayProfileTHR
cmsCreateInkLimitingDeviceLink           =    cmsCreateInkLimitingDeviceLink
cmsCreateInkLimitingDeviceLinkTHR        =    cmsCreateInkLimitingDeviceLinkTHR
cmsCreateLab2Profile                     =    cmsCreateLab2Profile
cmsCreateLab2ProfileTHR                  =    cmsCreateLab2ProfileTHR
cmsCreateLab4Profile                     =    cmsCreateLab4Profile
cmsCreateLab4ProfileTHR                  =    cmsCreateLab4ProfileTHR
cmsCreateLinearizationDeviceLink         =    cmsCreateLinearizationDeviceLink
cmsCreateLinearizationDeviceLinkTHR      =    cmsCreateLinearizationDeviceLinkTHR
cmsCreateMultiprofileTransform           =    cmsCreateMultiprofileTransform
cmsCreateMultiprofileTransformTHR        =    cmsCreateMultiprofileTransformTHR
cmsCreateNULLProfile                     =    cmsCreateNULLProfile
cmsCreateNULLProfileTHR                  =    cmsCreateNULLProfileTHR
cmsCreateProfilePlaceholder              =    cmsCreateProfilePlaceholder
cmsCreateProofingTransform               =    cmsCreateProofingTransform
cmsCreateProofingTransformTHR            =    cmsCreateProofingTransformTHR
cmsCreateRGBProfile                      =    cmsCreateRGBProfile
cmsCreateRGBProfileTHR                   =    cmsCreateRGBProfileTHR
cmsCreateTransform                       =    cmsCreateTransform
cmsCreateTransformTHR                    =    cmsCreateTransformTHR
cmsCreateXYZProfile                      =    cmsCreateXYZProfile
cmsCreateXYZProfileTHR                   =    cmsCreateXYZProfileTHR
cmsD50_xyY                               =    cmsD50_xyY
cmsD50_XYZ                               =    cmsD50_XYZ
_cmsDecodeDateTimeNumber                 =    _cmsDecodeDateTimeNumber
_cmsDefaultICCintents                    =    _cmsDefaultICCintents
cmsDeleteTransform                       =    cmsDeleteTransform
cmsDeltaE                                =    cmsDeltaE
cmsDetectBlackPoint                      =    cmsDetectBlackPoint
cmsDetectDestinationBlackPoint           =    cmsDetectDestinationBlackPoint
cmsDetectTAC                             =    cmsDetectTAC
cmsDesaturateLab                         =    cmsDesaturateLab
cmsDoTransform                           =    cmsDoTransform
cmsDoTransformStride                     =    cmsDoTransformStride
_cmsDoubleTo15Fixed16                    =    _cmsDoubleTo15Fixed16
_cmsDoubleTo8Fixed8                      =    _cmsDoubleTo8Fixed8
_cmsDupMem                               =    _cmsDupMem
cmsDupNamedColorList                     =    cmsDupNamedColorList
cmsDupProfileSequenceDescription         =    cmsDupProfileSequenceDescription
cmsDupToneCurve                          =    cmsDupToneCurve
_cmsEncodeDateTimeNumber                 =    _cmsEncodeDateTimeNumber
cmsEstimateGamma                         =    cmsEstimateGamma
cmsGetToneCurveEstimatedTableEntries     =    cmsGetToneCurveEstimatedTableEntries
cmsGetToneCurveEstimatedTable            =    cmsGetToneCurveEstimatedTable
cmsEvalToneCurve16                       =    cmsEvalToneCurve16
cmsEvalToneCurveFloat                    =    cmsEvalToneCurveFloat
cmsfilelength                            =    cmsfilelength
cmsFloat2LabEncoded                      =    cmsFloat2LabEncoded
cmsFloat2LabEncodedV2                    =    cmsFloat2LabEncodedV2
cmsFloat2XYZEncoded                      =    cmsFloat2XYZEncoded
cmsFormatterForColorspaceOfProfile       =    cmsFormatterForColorspaceOfProfile
cmsFormatterForPCSOfProfile              =    cmsFormatterForPCSOfProfile
_cmsFree                                 =    _cmsFree
cmsFreeNamedColorList                    =    cmsFreeNamedColorList
cmsFreeProfileSequenceDescription        =    cmsFreeProfileSequenceDescription
cmsFreeToneCurve                         =    cmsFreeToneCurve
cmsFreeToneCurveTriple                   =    cmsFreeToneCurveTriple
cmsGBDAlloc                              =    cmsGBDAlloc
cmsGBDFree                               =    cmsGBDFree
cmsGDBAddPoint                           =    cmsGDBAddPoint
cmsGDBCheckPoint                         =    cmsGDBCheckPoint
cmsGDBCompute                            =    cmsGDBCompute
cmsGetAlarmCodes                         =    cmsGetAlarmCodes
cmsGetColorSpace                         =    cmsGetColorSpace
cmsGetDeviceClass                        =    cmsGetDeviceClass
cmsGetEncodedICCversion                  =    cmsGetEncodedICCversion
cmsGetHeaderAttributes                   =    cmsGetHeaderAttributes
cmsGetHeaderCreationDateTime             =    cmsGetHeaderCreationDateTime
cmsGetHeaderFlags                        =    cmsGetHeaderFlags
cmsGetHeaderManufacturer                 =    cmsGetHeaderManufacturer
cmsGetHeaderModel                        =    cmsGetHeaderModel
cmsGetHeaderProfileID                    =    cmsGetHeaderProfileID
cmsGetHeaderRenderingIntent              =    cmsGetHeaderRenderingIntent
cmsGetNamedColorList                     =    cmsGetNamedColorList
cmsGetPCS                                =    cmsGetPCS
cmsGetPostScriptColorResource            =    cmsGetPostScriptColorResource
cmsGetPostScriptCRD                      =    cmsGetPostScriptCRD
cmsGetPostScriptCSA                      =    cmsGetPostScriptCSA
cmsGetProfileInfo                        =    cmsGetProfileInfo
cmsGetProfileInfoASCII                   =    cmsGetProfileInfoASCII
cmsGetProfileContextID                   =    cmsGetProfileContextID
cmsGetProfileVersion                     =    cmsGetProfileVersion
cmsGetSupportedIntents                   =    cmsGetSupportedIntents
cmsGetTagCount                           =    cmsGetTagCount
cmsGetTagSignature                       =    cmsGetTagSignature
cmsGetTransformContextID                 =    cmsGetTransformContextID
_cmsICCcolorSpace                        =    _cmsICCcolorSpace
_cmsIOPrintf                             =    _cmsIOPrintf
cmsIsCLUT                                =    cmsIsCLUT
cmsIsIntentSupported                     =    cmsIsIntentSupported
cmsIsMatrixShaper                        =    cmsIsMatrixShaper
cmsIsTag                                 =    cmsIsTag
cmsIsToneCurveDescending                 =    cmsIsToneCurveDescending
cmsIsToneCurveLinear                     =    cmsIsToneCurveLinear
cmsIsToneCurveMonotonic                  =    cmsIsToneCurveMonotonic
cmsIsToneCurveMultisegment               =    cmsIsToneCurveMultisegment
cmsGetToneCurveParametricType            =    cmsGetToneCurveParametricType
cmsIT8Alloc                              =    cmsIT8Alloc
cmsIT8DefineDblFormat                    =    cmsIT8DefineDblFormat
cmsIT8EnumDataFormat                     =    cmsIT8EnumDataFormat
cmsIT8EnumProperties                     =    cmsIT8EnumProperties
cmsIT8EnumPropertyMulti                  =    cmsIT8EnumPropertyMulti
cmsIT8Free                               =    cmsIT8Free
cmsIT8GetData                            =    cmsIT8GetData
cmsIT8GetDataDbl                         =    cmsIT8GetDataDbl
cmsIT8FindDataFormat                     =    cmsIT8FindDataFormat
cmsIT8GetDataRowCol                      =    cmsIT8GetDataRowCol
cmsIT8GetDataRowColDbl                   =    cmsIT8GetDataRowColDbl
cmsIT8GetPatchName                       =    cmsIT8GetPatchName
cmsIT8GetPatchByName                     =    cmsIT8GetPatchByName
cmsIT8GetProperty                        =    cmsIT8GetProperty
cmsIT8GetPropertyDbl                     =    cmsIT8GetPropertyDbl
cmsIT8GetPropertyMulti                   =    cmsIT8GetPropertyMulti
cmsIT8GetSheetType                       =    cmsIT8GetSheetType
cmsIT8LoadFromFile                       =    cmsIT8LoadFromFile
cmsIT8LoadFromMem                        =    cmsIT8LoadFromMem
cmsIT8SaveToFile                         =    cmsIT8SaveToFile
cmsIT8SaveToMem                          =    cmsIT8SaveToMem
cmsIT8SetComment                         =    cmsIT8SetComment
cmsIT8SetData                            =    cmsIT8SetData
cmsIT8SetDataDbl                         =    cmsIT8SetDataDbl
cmsIT8SetDataFormat                      =    cmsIT8SetDataFormat
cmsIT8SetDataRowCol                      =    cmsIT8SetDataRowCol
cmsIT8SetDataRowColDbl                   =    cmsIT8SetDataRowColDbl
cmsIT8SetPropertyDbl                     =    cmsIT8SetPropertyDbl
cmsIT8SetPropertyHex                     =    cmsIT8SetPropertyHex
cmsIT8SetPropertyStr                     =    cmsIT8SetPropertyStr
cmsIT8SetPropertyMulti                   =    cmsIT8SetPropertyMulti
cmsIT8SetPropertyUncooked                =    cmsIT8SetPropertyUncooked
cmsIT8SetSheetType                       =    cmsIT8SetSheetType
cmsIT8SetTable                           =    cmsIT8SetTable
cmsIT8SetTableByLabel                    =    cmsIT8SetTableByLabel
cmsIT8SetIndexColumn                     =    cmsIT8SetIndexColumn
cmsIT8TableCount                         =    cmsIT8TableCount
cmsJoinToneCurve                         =    cmsJoinToneCurve
cmsLab2LCh                               =    cmsLab2LCh
cmsLab2XYZ                               =    cmsLab2XYZ
cmsLabEncoded2Float                      =    cmsLabEncoded2Float
cmsLabEncoded2FloatV2                    =    cmsLabEncoded2FloatV2
cmsLCh2Lab                               =    cmsLCh2Lab
_cmsLCMScolorSpace                       =    _cmsLCMScolorSpace
cmsLinkTag                               =    cmsLinkTag
cmsTagLinkedTo                           =    cmsTagLinkedTo
cmsPipelineAlloc                         =    cmsPipelineAlloc
cmsPipelineCat                           =    cmsPipelineCat
cmsPipelineCheckAndRetreiveStages        =    cmsPipelineCheckAndRetreiveStages
cmsPipelineDup                           =    cmsPipelineDup
cmsPipelineStageCount                    =    cmsPipelineStageCount
cmsPipelineEval16                        =    cmsPipelineEval16
cmsPipelineEvalFloat                     =    cmsPipelineEvalFloat
cmsPipelineEvalReverseFloat              =    cmsPipelineEvalReverseFloat
cmsPipelineFree                          =    cmsPipelineFree
cmsPipelineGetPtrToFirstStage            =    cmsPipelineGetPtrToFirstStage
cmsPipelineGetPtrToLastStage             =    cmsPipelineGetPtrToLastStage
cmsPipelineInputChannels                 =    cmsPipelineInputChannels
cmsPipelineInsertStage                   =    cmsPipelineInsertStage
cmsPipelineOutputChannels                =    cmsPipelineOutputChannels
cmsPipelineSetSaveAs8bitsFlag            =    cmsPipelineSetSaveAs8bitsFlag
_cmsPipelineSetOptimizationParameters    =    _cmsPipelineSetOptimizationParameters
cmsPipelineUnlinkStage                   =    cmsPipelineUnlinkStage
_cmsMalloc                               =    _cmsMalloc
_cmsMallocZero                           =    _cmsMallocZero
_cmsMAT3eval                             =    _cmsMAT3eval
_cmsMAT3identity                         =    _cmsMAT3identity
_cmsMAT3inverse                          =    _cmsMAT3inverse
_cmsMAT3isIdentity                       =    _cmsMAT3isIdentity
_cmsMAT3per                              =    _cmsMAT3per
_cmsMAT3solve                            =    _cmsMAT3solve
cmsMD5computeID                          =    cmsMD5computeID
cmsMLUalloc                              =    cmsMLUalloc
cmsMLUdup                                =    cmsMLUdup
cmsMLUfree                               =    cmsMLUfree
cmsMLUgetASCII                           =    cmsMLUgetASCII
cmsMLUgetTranslation                     =    cmsMLUgetTranslation
cmsMLUgetWide                            =    cmsMLUgetWide
cmsMLUsetASCII                           =    cmsMLUsetASCII
cmsMLUsetWide                            =    cmsMLUsetWide
cmsStageAllocCLut16bit                   =    cmsStageAllocCLut16bit
cmsStageAllocCLut16bitGranular           =    cmsStageAllocCLut16bitGranular
cmsStageAllocCLutFloat                   =    cmsStageAllocCLutFloat
cmsStageAllocCLutFloatGranular           =    cmsStageAllocCLutFloatGranular
cmsStageAllocToneCurves                  =    cmsStageAllocToneCurves
cmsStageAllocIdentity                    =    cmsStageAllocIdentity
cmsStageAllocMatrix                      =    cmsStageAllocMatrix
_cmsStageAllocPlaceholder                =    _cmsStageAllocPlaceholder
cmsStageDup                              =    cmsStageDup
cmsStageFree                             =    cmsStageFree
cmsStageNext                             =    cmsStageNext
cmsStageInputChannels                    =    cmsStageInputChannels
cmsStageOutputChannels                   =    cmsStageOutputChannels
cmsStageSampleCLut16bit                  =    cmsStageSampleCLut16bit
cmsStageSampleCLutFloat                  =    cmsStageSampleCLutFloat
cmsStageType                             =    cmsStageType
cmsStageData                             =    cmsStageData
cmsNamedColorCount                       =    cmsNamedColorCount
cmsNamedColorIndex                       =    cmsNamedColorIndex
cmsNamedColorInfo                        =    cmsNamedColorInfo
cmsOpenIOhandlerFromFile                 =    cmsOpenIOhandlerFromFile
cmsOpenIOhandlerFromMem                  =    cmsOpenIOhandlerFromMem
cmsOpenIOhandlerFromNULL                 =    cmsOpenIOhandlerFromNULL
cmsOpenIOhandlerFromStream               =    cmsOpenIOhandlerFromStream
cmsOpenProfileFromFile                   =    cmsOpenProfileFromFile
cmsOpenProfileFromFileTHR                =    cmsOpenProfileFromFileTHR
cmsOpenProfileFromIOhandlerTHR           =    cmsOpenProfileFromIOhandlerTHR
cmsOpenProfileFromMem                    =    cmsOpenProfileFromMem
cmsOpenProfileFromMemTHR                 =    cmsOpenProfileFromMemTHR
cmsOpenProfileFromStream                 =    cmsOpenProfileFromStream
cmsOpenProfileFromStreamTHR              =    cmsOpenProfileFromStreamTHR
cmsPlugin                                =    cmsPlugin
_cmsRead15Fixed16Number                  =    _cmsRead15Fixed16Number
_cmsReadAlignment                        =    _cmsReadAlignment
_cmsReadFloat32Number                    =    _cmsReadFloat32Number
cmsReadRawTag                            =    cmsReadRawTag
cmsReadTag                               =    cmsReadTag
_cmsReadTypeBase                         =    _cmsReadTypeBase
_cmsReadUInt16Array                      =    _cmsReadUInt16Array
_cmsReadUInt16Number                     =    _cmsReadUInt16Number
_cmsReadUInt32Number                     =    _cmsReadUInt32Number
_cmsReadUInt64Number                     =    _cmsReadUInt64Number
_cmsReadUInt8Number                      =    _cmsReadUInt8Number
_cmsReadXYZNumber                        =    _cmsReadXYZNumber
_cmsRealloc                              =    _cmsRealloc
cmsReverseToneCurve                      =    cmsReverseToneCurve
cmsReverseToneCurveEx                    =    cmsReverseToneCurveEx
cmsSaveProfileToFile                     =    cmsSaveProfileToFile
cmsSaveProfileToIOhandler                =    cmsSaveProfileToIOhandler
cmsSaveProfileToMem                      =    cmsSaveProfileToMem
cmsSaveProfileToStream                   =    cmsSaveProfileToStream
cmsSetAdaptationState                    =    cmsSetAdaptationState
cmsSetAlarmCodes                         =    cmsSetAlarmCodes
cmsSetColorSpace                         =    cmsSetColorSpace
cmsSetDeviceClass                        =    cmsSetDeviceClass
cmsSetEncodedICCversion                  =    cmsSetEncodedICCversion
cmsSetHeaderAttributes                   =    cmsSetHeaderAttributes
cmsSetHeaderFlags                        =    cmsSetHeaderFlags
cmsSetHeaderManufacturer                 =    cmsSetHeaderManufacturer
cmsSetHeaderModel                        =    cmsSetHeaderModel
cmsSetHeaderProfileID                    =    cmsSetHeaderProfileID
cmsSetHeaderRenderingIntent              =    cmsSetHeaderRenderingIntent
cmsSetLogErrorHandler                    =    cmsSetLogErrorHandler
cmsSetPCS                                =    cmsSetPCS
cmsSetProfileVersion                     =    cmsSetProfileVersion
cmsSignalError                           =    cmsSignalError
cmsSmoothToneCurve                       =    cmsSmoothToneCurve
cmsstrcasecmp                            =    cmsstrcasecmp
cmsTempFromWhitePoint                    =    cmsTempFromWhitePoint
cmsTransform2DeviceLink                  =    cmsTransform2DeviceLink
cmsUnregisterPlugins                     =    cmsUnregisterPlugins
_cmsVEC3cross                            =    _cmsVEC3cross
_cmsVEC3distance                         =    _cmsVEC3distance
_cmsVEC3dot                              =    _cmsVEC3dot
_cmsVEC3init                             =    _cmsVEC3init
_cmsVEC3length                           =    _cmsVEC3length
_cmsVEC3minus                            =    _cmsVEC3minus
cmsWhitePointFromTemp                    =    cmsWhitePointFromTemp
_cmsWrite15Fixed16Number                 =    _cmsWrite15Fixed16Number
_cmsWriteAlignment                       =    _cmsWriteAlignment
_cmsWriteFloat32Number                   =    _cmsWriteFloat32Number
cmsWriteRawTag                           =    cmsWriteRawTag
cmsWriteTag                              =    cmsWriteTag
_cmsWriteTypeBase                        =    _cmsWriteTypeBase
_cmsWriteUInt16Array                     =    _cmsWriteUInt16Array
_cmsWriteUInt16Number                    =    _cmsWriteUInt16Number
_cmsWriteUInt32Number                    =    _cmsWriteUInt32Number
_cmsWriteUInt64Number                    =    _cmsWriteUInt64Number
_cmsWriteUInt8Number                     =    _cmsWriteUInt8Number
_cmsWriteXYZNumber                       =    _cmsWriteXYZNumber
cmsxyY2XYZ                               =   cmsxyY2XYZ
cmsXYZ2Lab                               =   cmsXYZ2Lab
cmsXYZ2xyY                               =   cmsXYZ2xyY
cmsXYZEncoded2Float                      =   cmsXYZEncoded2Float
cmsSliceSpace16                          =   cmsSliceSpace16
cmsSliceSpaceFloat                       =   cmsSliceSpaceFloat
cmsChangeBuffersFormat                   =   cmsChangeBuffersFormat
cmsDictAlloc                             =   cmsDictAlloc
cmsDictFree                              =   cmsDictFree
cmsDictDup                               =   cmsDictDup
cmsDictAddEntry                          =   cmsDictAddEntry
cmsDictGetEntryList                      =   cmsDictGetEntryList
cmsDictNextEntry                         =   cmsDictNextEntry
_cmsGetTransformUserData                 =   _cmsGetTransformUserData
_cmsSetTransformUserData                 =   _cmsSetTransformUserData
_cmsGetTransformFormatters16             =   _cmsGetTransformFormatters16
_cmsGetTransformFormattersFloat          =   _cmsGetTransformFormattersFloat
cmsGetHeaderCreator                      =   cmsGetHeaderCreator
cmsPluginTHR                             =   cmsPluginTHR
cmsGetPipelineContextID                  =   cmsGetPipelineContextID
cmsGetTransformInputFormat               =   cmsGetTransformInputFormat
cmsGetTransformOutputFormat              =   cmsGetTransformOutputFormat
cmsCreateContext                         =   cmsCreateContext            
cmsDupContext                            =   cmsDupContext               
cmsDeleteContext                         =   cmsDeleteContext              
cmsGetContextUserData                    =   cmsGetContextUserData       
cmsUnregisterPluginsTHR                  =   cmsUnregisterPluginsTHR 
cmsSetAlarmCodesTHR                      =   cmsSetAlarmCodesTHR     
cmsGetAlarmCodesTHR                      =   cmsGetAlarmCodesTHR
cmsSetAdaptationStateTHR                 =   cmsSetAdaptationStateTHR
cmsSetLogErrorHandlerTHR                 =   cmsSetLogErrorHandlerTHR
cmsGetSupportedIntentsTHR                =   cmsGetSupportedIntentsTHR
cmsMLUtranslationsCount                  =   cmsMLUtranslationsCount
cmsMLUtranslationsCodes                  =   cmsMLUtranslationsCodes
_cmsCreateMutex                          =   _cmsCreateMutex 
_cmsDestroyMutex                         =   _cmsDestroyMutex
_cmsLockMutex                            =   _cmsLockMutex   
_cmsUnlockMutex                          =   _cmsUnlockMutex 
cmsGetProfileIOhandler                   =   cmsGetProfileIOhandler
cmsGetEncodedCMMversion                  =   cmsGetEncodedCMMversion
